@@ -3,16 +3,17 @@ import asyncio.tasks
 import datetime
 import functools
 import ipaddress
+import itertools
 import typing
 from typing import Dict, List, Optional, Union
 
 from aiohttp import web
+from aiohttp.web_exceptions import HTTPGone
 
 from .. import types
 from ..bot import api
 from ..types import ParseMode
 from ..types.base import Boolean, Float, Integer, String
-from ..utils import context
 from ..utils import helper, markdown
 from ..utils import json
 from ..utils.deprecated import warn_deprecated as warn
@@ -20,6 +21,7 @@ from ..utils.exceptions import TimeoutWarning
 from ..utils.payload import prepare_arg
 
 DEFAULT_WEB_PATH = '/webhook'
+DEFAULT_ROUTE_NAME = 'webhook_handler'
 BOT_DISPATCHER_KEY = 'BOT_DISPATCHER'
 
 RESPONSE_TIMEOUT = 55
@@ -28,8 +30,8 @@ WEBHOOK = 'webhook'
 WEBHOOK_CONNECTION = 'WEBHOOK_CONNECTION'
 WEBHOOK_REQUEST = 'WEBHOOK_REQUEST'
 
-TELEGRAM_IP_LOWER = ipaddress.IPv4Address('149.154.167.197')
-TELEGRAM_IP_UPPER = ipaddress.IPv4Address('149.154.167.233')
+TELEGRAM_SUBNET_1 = ipaddress.IPv4Network('149.154.160.0/20')
+TELEGRAM_SUBNET_2 = ipaddress.IPv4Network('91.108.4.0/22')
 
 allowed_ips = set()
 
@@ -45,18 +47,26 @@ def _check_ip(ip: str) -> bool:
     return address in allowed_ips
 
 
-def allow_ip(*ips: str):
+def allow_ip(*ips: typing.Union[str, ipaddress.IPv4Network, ipaddress.IPv4Address]):
     """
     Allow ip address.
 
     :param ips:
     :return:
     """
-    allowed_ips.update(ipaddress.IPv4Address(ip) for ip in ips)
+    for ip in ips:
+        if isinstance(ip, ipaddress.IPv4Address):
+            allowed_ips.add(ip)
+        elif isinstance(ip, str):
+            allowed_ips.add(ipaddress.IPv4Address(ip))
+        elif isinstance(ip, ipaddress.IPv4Network):
+            allowed_ips.update(ip.hosts())
+        else:
+            raise ValueError(f"Bad type of ipaddress: {type(ip)} ('{ip}')")
 
 
 # Allow access from Telegram servers
-allow_ip(*(ip for ip in range(int(TELEGRAM_IP_LOWER), int(TELEGRAM_IP_UPPER) + 1)))
+allow_ip(TELEGRAM_SUBNET_1, TELEGRAM_SUBNET_2)
 
 
 class WebhookRequestHandler(web.View):
@@ -88,8 +98,9 @@ class WebhookRequestHandler(web.View):
         """
         dp = self.request.app[BOT_DISPATCHER_KEY]
         try:
-            context.set_value('dispatcher', dp)
-            context.set_value('bot', dp.bot)
+            from aiogram import Bot, Dispatcher
+            Dispatcher.set_current(dp)
+            Bot.set_current(dp.bot)
         except RuntimeError:
             pass
         return dp
@@ -116,9 +127,9 @@ class WebhookRequestHandler(web.View):
         """
         self.validate_ip()
 
-        context.update_state({'CALLER': WEBHOOK,
-                              WEBHOOK_CONNECTION: True,
-                              WEBHOOK_REQUEST: self.request})
+        # context.update_state({'CALLER': WEBHOOK,
+        #                       WEBHOOK_CONNECTION: True,
+        #                       WEBHOOK_REQUEST: self.request})
 
         dispatcher = self.get_dispatcher()
         update = await self.parse_update(dispatcher.bot)
@@ -127,8 +138,14 @@ class WebhookRequestHandler(web.View):
         response = self.get_response(results)
 
         if response:
-            return response.get_web_response()
-        return web.Response(text='ok')
+            web_response = response.get_web_response()
+        else:
+            web_response = web.Response(text='ok')
+
+        if self.request.app.get('RETRY_AFTER', None):
+            web_response.headers['Retry-After'] = self.request.app['RETRY_AFTER']
+
+        return web_response
 
     async def get(self):
         self.validate_ip()
@@ -156,7 +173,7 @@ class WebhookRequestHandler(web.View):
         timeout_handle = loop.call_later(RESPONSE_TIMEOUT, asyncio.tasks._release_waiter, waiter)
         cb = functools.partial(asyncio.tasks._release_waiter, waiter)
 
-        fut = asyncio.ensure_future(dispatcher.process_update(update), loop=loop)
+        fut = asyncio.ensure_future(dispatcher.updates_handler.notify(update), loop=loop)
         fut.add_done_callback(cb)
 
         try:
@@ -170,7 +187,7 @@ class WebhookRequestHandler(web.View):
             if fut.done():
                 return fut.result()
             else:
-                context.set_value(WEBHOOK_CONNECTION, False)
+                # context.set_value(WEBHOOK_CONNECTION, False)
                 fut.remove_done_callback(cb)
                 fut.add_done_callback(self.respond_via_request)
         finally:
@@ -195,7 +212,7 @@ class WebhookRequestHandler(web.View):
             results = task.result()
         except Exception as e:
             loop.create_task(
-                dispatcher.errors_handlers.notify(dispatcher, context.get_value('update_object'), e))
+                dispatcher.errors_handlers.notify(dispatcher, types.Update.get_current(), e))
         else:
             response = self.get_response(results)
             if response is not None:
@@ -210,7 +227,7 @@ class WebhookRequestHandler(web.View):
         """
         if results is None:
             return None
-        for result in results:
+        for result in itertools.chain.from_iterable(results):
             if isinstance(result, BaseResponse):
                 return result
 
@@ -242,19 +259,33 @@ class WebhookRequestHandler(web.View):
             ip_address, accept = self.check_ip()
             if not accept:
                 raise web.HTTPUnauthorized()
-            context.set_value('TELEGRAM_IP', ip_address)
+            # context.set_value('TELEGRAM_IP', ip_address)
 
 
-def configure_app(dispatcher, app: web.Application, path=DEFAULT_WEB_PATH):
+class GoneRequestHandler(web.View):
+    """
+    If a webhook returns the HTTP error 410 Gone for all requests for more than 23 hours successively,
+    it can be automatically removed.
+    """
+
+    async def get(self):
+        raise HTTPGone()
+
+    async def post(self):
+        raise HTTPGone()
+
+
+def configure_app(dispatcher, app: web.Application, path=DEFAULT_WEB_PATH, route_name=DEFAULT_ROUTE_NAME):
     """
     You can prepare web.Application for working with webhook handler.
 
     :param dispatcher: Dispatcher instance
     :param app: :class:`aiohttp.web.Application`
     :param path: Path to your webhook.
+    :param route_name: Name of webhook handler route
     :return:
     """
-    app.router.add_route('*', path, WebhookRequestHandler, name='webhook_handler')
+    app.router.add_route('*', path, WebhookRequestHandler, name=route_name)
     app[BOT_DISPATCHER_KEY] = dispatcher
 
 
@@ -332,8 +363,8 @@ class BaseResponse:
 
     async def __call__(self, bot=None):
         if bot is None:
-            from aiogram.dispatcher import ctx
-            bot = ctx.get_bot()
+            from aiogram import Bot
+            bot = Bot.get_current()
         return await self.execute_response(bot)
 
     async def __aenter__(self):
@@ -426,7 +457,8 @@ class ParseModeMixin:
 
         :return:
         """
-        bot = context.get_value('bot', None)
+        from aiogram import Bot
+        bot = Bot.get_current()
         if bot is not None:
             return bot.parse_mode
 
@@ -932,7 +964,7 @@ class SendMediaGroup(BaseResponse, ReplyToMixin, DisableNotificationMixin):
         self.reply_to_message_id = reply_to_message_id
 
     def prepare(self):
-        files = self.media.get_files()
+        files = dict(self.media.get_files())
         if files:
             raise TypeError('Allowed only file ID or URL\'s')
 
