@@ -1,9 +1,14 @@
+from __future__ import annotations
+
 import io
 import typing
 from typing import TypeVar
 
+from babel.support import LazyProxy
+
 from .fields import BaseField
 from ..utils import json
+from ..utils.mixins import ContextInstanceMixin
 
 __all__ = ('MetaTelegramObject', 'TelegramObject', 'InputFile', 'String', 'Integer', 'Float', 'Boolean')
 
@@ -53,6 +58,7 @@ class MetaTelegramObject(type):
         setattr(cls, ALIASES_ATTR_NAME, aliases)
 
         mcs._objects[cls.__name__] = cls
+
         return cls
 
     @property
@@ -60,7 +66,7 @@ class MetaTelegramObject(type):
         return cls._objects
 
 
-class TelegramObject(metaclass=MetaTelegramObject):
+class TelegramObject(ContextInstanceMixin, metaclass=MetaTelegramObject):
     """
     Abstract class for telegram objects
     """
@@ -137,8 +143,14 @@ class TelegramObject(metaclass=MetaTelegramObject):
 
     @property
     def bot(self):
-        from ..dispatcher import ctx
-        return ctx.get_bot()
+        from ..bot.bot import Bot
+
+        bot = Bot.get_current()
+        if bot is None:
+            raise RuntimeError("Can't get bot instance from context. "
+                               "You can fix it with setting current instance: "
+                               "'Bot.set_current(bot_instance)'")
+        return bot
 
     def to_python(self) -> typing.Dict:
         """
@@ -153,6 +165,8 @@ class TelegramObject(metaclass=MetaTelegramObject):
                 value = self.props[name].export(self)
             if isinstance(value, TelegramObject):
                 value = value.to_python()
+            if isinstance(value, LazyProxy):
+                value = str(value)
             result[self.props_aliases.get(name, name)] = value
         return result
 
