@@ -4,11 +4,10 @@ for more information https://core.telegram.org/widgets/login#checking-authorizat
 
 Source: https://gist.github.com/JrooTJunior/887791de7273c9df5277d2b1ecadc839
 """
+import collections
 import hashlib
 import hmac
 
-import collections
-
 
 def generate_hash(data: dict, token: str) -> str:
     """
@@ -21,7 +20,7 @@ def generate_hash(data: dict, token: str) -> str:
     secret = hashlib.sha256()
     secret.update(token.encode('utf-8'))
     sorted_params = collections.OrderedDict(sorted(data.items()))
-    msg = '\n'.join(["{}={}".format(k, v) for k, v in sorted_params.items() if k != 'hash'])
+    msg = '\n'.join("{}={}".format(k, v) for k, v in sorted_params.items() if k != 'hash')
     return hmac.new(secret.digest(), msg.encode('utf-8'), digestmod=hashlib.sha256).hexdigest()
 
 
