@@ -2,13 +2,15 @@ import asyncio
 import datetime
 import functools
 import secrets
+from typing import Callable, Union, Optional, Any
 from warnings import warn
 
 from aiohttp import web
+from aiohttp.web_app import Application
 
-from . import context
 from ..bot.api import log
-from ..dispatcher.webhook import BOT_DISPATCHER_KEY, WebhookRequestHandler
+from ..dispatcher.dispatcher import Dispatcher
+from ..dispatcher.webhook import BOT_DISPATCHER_KEY, DEFAULT_ROUTE_NAME, WebhookRequestHandler
 
 APP_EXECUTOR_KEY = 'APP_EXECUTOR'
 
@@ -21,7 +23,7 @@ def _setup_callbacks(executor, on_startup=None, on_shutdown=None):
 
 
 def start_polling(dispatcher, *, loop=None, skip_updates=False, reset_webhook=True,
-                  on_startup=None, on_shutdown=None, timeout=None):
+                  on_startup=None, on_shutdown=None, timeout=20, fast=True):
     """
     Start bot in long-polling mode
 
@@ -36,11 +38,40 @@ def start_polling(dispatcher, *, loop=None, skip_updates=False, reset_webhook=Tr
     executor = Executor(dispatcher, skip_updates=skip_updates, loop=loop)
     _setup_callbacks(executor, on_startup, on_shutdown)
 
-    executor.start_polling(reset_webhook=reset_webhook, timeout=timeout)
+    executor.start_polling(reset_webhook=reset_webhook, timeout=timeout, fast=fast)
+
+
+def set_webhook(dispatcher: Dispatcher, webhook_path: str, *, loop: Optional[asyncio.AbstractEventLoop] = None,
+                skip_updates: bool = None, on_startup: Optional[Callable] = None,
+                on_shutdown: Optional[Callable] = None, check_ip: bool = False,
+                retry_after: Optional[Union[str, int]] = None, route_name: str = DEFAULT_ROUTE_NAME,
+                web_app: Optional[Application] = None):
+    """
+    Set webhook for bot
+
+    :param dispatcher: Dispatcher
+    :param webhook_path: str
+    :param loop: Optional[asyncio.AbstractEventLoop] (default: None)
+    :param skip_updates: bool (default: None)
+    :param on_startup: Optional[Callable] (default: None)
+    :param on_shutdown: Optional[Callable] (default: None)
+    :param check_ip: bool (default: False)
+    :param retry_after: Optional[Union[str, int]] See https://tools.ietf.org/html/rfc7231#section-7.1.3 (default: None)
+    :param route_name: str (default: 'webhook_handler')
+    :param web_app: Optional[Application] (default: None)
+    :return:
+    """
+    executor = Executor(dispatcher, skip_updates=skip_updates, check_ip=check_ip, retry_after=retry_after,
+                        loop=loop)
+    _setup_callbacks(executor, on_startup, on_shutdown)
+
+    executor.set_webhook(webhook_path, route_name=route_name, web_app=web_app)
+    return executor
 
 
 def start_webhook(dispatcher, webhook_path, *, loop=None, skip_updates=None,
-                  on_startup=None, on_shutdown=None, check_ip=False, **kwargs):
+                  on_startup=None, on_shutdown=None, check_ip=False, retry_after=None, route_name=DEFAULT_ROUTE_NAME,
+                  **kwargs):
     """
     Start bot in webhook mode
 
@@ -51,13 +82,20 @@ def start_webhook(dispatcher, webhook_path, *, loop=None, skip_updates=None,
     :param on_startup:
     :param on_shutdown:
     :param check_ip:
+    :param route_name:
     :param kwargs:
     :return:
     """
-    executor = Executor(dispatcher, skip_updates=skip_updates, check_ip=check_ip, loop=loop)
-    _setup_callbacks(executor, on_startup, on_shutdown)
-
-    executor.start_webhook(webhook_path, **kwargs)
+    executor = set_webhook(dispatcher=dispatcher,
+                           webhook_path=webhook_path,
+                           loop=loop,
+                           skip_updates=skip_updates,
+                           on_startup=on_startup,
+                           on_shutdown=on_shutdown,
+                           check_ip=check_ip,
+                           retry_after=retry_after,
+                           route_name=route_name)
+    executor.run_app(**kwargs)
 
 
 def start(dispatcher, future, *, loop=None, skip_updates=None,
@@ -84,12 +122,13 @@ class Executor:
     Main executor class
     """
 
-    def __init__(self, dispatcher, skip_updates=None, check_ip=False, loop=None):
+    def __init__(self, dispatcher, skip_updates=None, check_ip=False, retry_after=None, loop=None):
         if loop is None:
             loop = dispatcher.loop
         self.dispatcher = dispatcher
         self.skip_updates = skip_updates
         self.check_ip = check_ip
+        self.retry_after = retry_after
         self.loop = loop
 
         self._identity = secrets.token_urlsafe(16)
@@ -102,6 +141,10 @@ class Executor:
 
         self._freeze = False
 
+        from aiogram import Bot, Dispatcher
+        Bot.set_current(dispatcher.bot)
+        Dispatcher.set_current(dispatcher)
+
     @property
     def frozen(self):
         return self._freeze
@@ -174,30 +217,37 @@ class Executor:
         self._check_frozen()
         self._freeze = True
 
-        self.loop.set_task_factory(context.task_factory)
+        # self.loop.set_task_factory(context.task_factory)
 
-    def _prepare_webhook(self, path=None, handler=WebhookRequestHandler):
+    def _prepare_webhook(self, path=None, handler=WebhookRequestHandler, route_name=DEFAULT_ROUTE_NAME, app=None):
         self._check_frozen()
         self._freeze = True
 
-        self.loop.set_task_factory(context.task_factory)
+        # self.loop.set_task_factory(context.task_factory)
 
-        app = self._web_app
-        if app is None:
+        if app is not None:
+            self._web_app = app
+        elif self._web_app is None:
             self._web_app = app = web.Application()
+        else:
+            raise RuntimeError("web.Application() is already configured!")
+
+        if self.retry_after:
+            app['RETRY_AFTER'] = self.retry_after
 
         if self._identity == app.get(self._identity):
             # App is already configured
             return
 
         if path is not None:
-            app.router.add_route('*', path, handler, name='webhook_handler')
+            app.router.add_route('*', path, handler, name=route_name)
 
         async def _wrap_callback(cb, _):
             return await cb(self.dispatcher)
 
         for callback in self._on_startup_webhook:
             app.on_startup.append(functools.partial(_wrap_callback, callback))
+
         # for callback in self._on_shutdown_webhook:
         #     app.on_shutdown.append(functools.partial(_wrap_callback, callback))
 
@@ -210,21 +260,38 @@ class Executor:
         app[self._identity] = datetime.datetime.now()
         app['_check_ip'] = self.check_ip
 
-    def start_webhook(self, webhook_path=None, request_handler=WebhookRequestHandler, **kwargs):
+    def set_webhook(self, webhook_path: Optional[str] = None, request_handler: Any = WebhookRequestHandler,
+                    route_name: str = DEFAULT_ROUTE_NAME, web_app: Optional[Application] = None):
+        """
+        Set webhook for bot
+
+        :param webhook_path: Optional[str] (default: None)
+        :param request_handler: Any (default: WebhookRequestHandler)
+        :param route_name: str Name of webhook handler route (default: 'webhook_handler')
+        :param web_app: Optional[Application] (default: None)
+        :return:
+        """
+        self._prepare_webhook(webhook_path, request_handler, route_name, web_app)
+        self.loop.run_until_complete(self._startup_webhook())
+
+    def run_app(self, **kwargs):
+        web.run_app(self._web_app, **kwargs)
+
+    def start_webhook(self, webhook_path=None, request_handler=WebhookRequestHandler, route_name=DEFAULT_ROUTE_NAME,
+                      **kwargs):
         """
         Start bot in webhook mode
 
         :param webhook_path:
         :param request_handler:
+        :param route_name: Name of webhook handler route
         :param kwargs:
         :return:
         """
-        self._prepare_webhook(webhook_path, request_handler)
-        self.loop.run_until_complete(self._startup_webhook())
+        self.set_webhook(webhook_path=webhook_path, request_handler=request_handler, route_name=route_name)
+        self.run_app(**kwargs)
 
-        web.run_app(self._web_app, **kwargs)
-
-    def start_polling(self, reset_webhook=None, timeout=None):
+    def start_polling(self, reset_webhook=None, timeout=20, fast=True):
         """
         Start bot in long-polling mode
 
@@ -236,14 +303,14 @@ class Executor:
 
         try:
             loop.run_until_complete(self._startup_polling())
-            loop.create_task(self.dispatcher.start_polling(reset_webhook=reset_webhook, timeout=timeout))
+            loop.create_task(self.dispatcher.start_polling(reset_webhook=reset_webhook, timeout=timeout, fast=fast))
             loop.run_forever()
         except (KeyboardInterrupt, SystemExit):
             # loop.stop()
             pass
         finally:
             loop.run_until_complete(self._shutdown_polling())
-        log.warning("Goodbye!")
+            log.warning("Goodbye!")
 
     def start(self, future):
         """
@@ -266,15 +333,13 @@ class Executor:
             loop.stop()
         finally:
             loop.run_until_complete(self._shutdown_polling())
-        log.warning("Goodbye!")
+            log.warning("Goodbye!")
         return result
 
     async def _skip_updates(self):
         await self.dispatcher.reset_webhook(True)
-        count = await self.dispatcher.skip_updates()
-        if count:
-            log.warning(f"Skipped {count} updates.")
-        return count
+        await self.dispatcher.skip_updates()
+        log.warning(f"Updates are skipped successfully.")
 
     async def _welcome(self):
         user = await self.dispatcher.bot.me
